/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.util.List;
import java.util.ArrayList;

/**
 * HybsTimerTaskManager.java は、HybsTimerTask オブジェクトを管理しているクラスです。
 *
 * HybsTimerTask は、タスクとして一定間隔で実行されます。オブジェクトが生成されてから、
 * 実行中のタスクを、このクラスで管理することにより、個別に削除、生成、全件終了等の制御が
 * 可能になります。
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class HybsTimerTaskManager implements Cleanable {
	private final List<HybsTimerTask> list = new ArrayList<HybsTimerTask>();

	/**
	 * HybsTimerTask クラスの 実オブジェクトを管理のため、登録します。
	 *
	 * @param task HybsTimerTaskオブジェクト
	 */
	public synchronized void addTask( final HybsTimerTask task ) {
		list.add( task );
	}

	/**
	 * このタイマータスクオブジェクトを削除します。
	 * このとき、すでにタイマータスク自信でキャンセルされているオブジェクトの
	 * 削除も行います。
	 *
	 * @param key  HybsTimerTask のユニークキー
	 *
	 * @return 結果(削除成功 true/ 削除できなかった false )
	 */
	public synchronized boolean cancelTask( final int key ) {
		boolean flag = false;

		int size = list.size();
		for( int i=size-1; i>=0; i-- ) {
			HybsTimerTask task = list.get( i );
			if( task != null ) {
				if( !task.isAlive() || ( task.getUniqKey() == key ) ) {
					list.remove( i );
					task.cancel();
					flag = true;
				}
			}
			else {
				list.remove( i );
			}
		}

		return flag;
	}

	/**
	 * このタイマータスクオブジェクトを削除します。
	 *
	 * @param  name  HybsTimerTask の名前
	 *
	 * @return 結果(削除成功 true/ 削除できなかった false )
	 */
	public synchronized boolean cancelTask( final String name ) {
		boolean flag = false;

		if( name == null ) { return flag; }

		int size = list.size();
		for( int i=size-1; i>=0; i-- ) {
			HybsTimerTask task = list.get( i );
			if( task != null ) {
				if( !task.isAlive() || ( name.equals( task.getName() ) ) ) {
					list.remove( i );
					task.cancel();
					flag = true;
				}
			}
			else {
				list.remove( i );
			}
		}

		return flag;
	}

	/**
	 * 現在実行中の全てのタイマータスクオブジェクトを削除します。
	 * (org.opengion.fukurou.util.Cleanable の実装)。
	 *
	 * @og.rev 4.0.0.0 (2005/01/31) 新規追加
	 */
	public synchronized void clear() {
		HybsTimerTask[] tasks = list.toArray( new HybsTimerTask[list.size()] );

		for( int i=0; i<tasks.length; i++ ) {
			if( tasks[i] != null ) {
				tasks[i].cancel();
				tasks[i] = null;
			}
		}
		list.clear();
	}

	/**
	 * 現在実行中の、タイマータスクを、すべて返します。
	 *
	 * @return タイマータスクの配列
	 */
	public synchronized HybsTimerTask[] toArray() {
		int size = list.size();
		for( int i=size-1; i>=0; i-- ) {
			HybsTimerTask task = list.get( i );
			if( task != null ) {
				if( !task.isAlive() ) {
					list.remove( i );
				}
			}
			else {
				list.remove( i );
			}
		}

		HybsTimerTask[] tasks = list.toArray( new HybsTimerTask[list.size()] );

		return tasks;
	}

	/**
	 * 現在実行中の、タイマータスクの件数を返します。
	 *
	 * タスクがすでに 終了していたり、null 化されているかをチェックし、
	 * 不要なタスクは、削除します。
	 *
	 * @return タイマータスクの件数
	 */
	public synchronized int size() {
		int size = list.size();
		for( int i=size-1; i>=0; i-- ) {
			HybsTimerTask task = list.get( i );
			if( task != null ) {
				if( !task.isAlive() ) {
					list.remove( i );
				}
			}
			else {
				list.remove( i );
			}
		}

		return list.size();
	}

	/**
	 * 現在実行中の、タイマータスクに、同一の Name 属性を持ったタスクが存在するかどうか。
	 *
	 * @param  nm タイマータスクの、Name 属性
	 *
	 * @return 存在する：true / 存在しない：false
	 */
	public synchronized boolean contains( final String nm ) {
		if( nm == null ) { return false; }

		HybsTimerTask[] tasks = toArray() ;

		for( int i=0; i<tasks.length; i++ ) {
			if( tasks[i] != null && nm.equals( tasks[i].getName() ) ) { return true; }
		}
		return false;
	}
}
